/*
 * Licensed Materials - Property of IBM                               
 * Rational Software Architect RealTime Edition - TARGET RTS                  
 * Copyright IBM Corporation 1999, 2014. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTProtocol_inl__
#define __RTProtocol_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTFieldOffset_h__
#include <RTFieldOffset.h>
#endif

#ifndef __RTGlobalSignal_h__
#include <RTGlobalSignal.h>
#endif

#ifndef __RTPortDescriptor_h__
#include <RTPortDescriptor.h>
#endif

#ifndef __RTProtocol_h__
#include <RTProtocol.h>
#endif

#ifndef __RTProtocolDescriptor_h__
#include <RTProtocolDescriptor.h>
#endif

#ifndef __RTTypedValue_h__
#include <RTTypedValue.h>
#endif

RTS_INLINE RTProtocol::RTProtocol( void )
    : _ends ( (End                         *)0 )
    , _info ( (const RTPortDescriptor      *)0 )
    , _flags( SendTypeCheck | ReceiveTypeCheck )
    , _layer( (RTLayerData                 *)0 )
{
}

RTS_INLINE int RTProtocol::getId( void ) const
{
    return (int)_info->id;
}

RTS_INLINE const char * RTProtocol::getName( void ) const
{
    return _info->name;
}

RTS_INLINE RTActor * RTProtocol::getOwner( void ) const
{
    return RTObjectOf( RTActor,
		       - _info->offset,
		       RTConstCast( RTProtocol, this ) );
}

RTS_INLINE int RTProtocol::size( void ) const
{
    return _info->replication;
}

RTS_INLINE int RTProtocol::getFlags( void ) const
{
    return _flags;
}

RTS_INLINE int RTProtocol::isRegistered( void ) const
{
    return _layer != (RTLayerData *)0;
}

RTS_INLINE int RTProtocol::bindingNotificationRequested( void ) const
{
    return ( getFlags() & BindingNotification ) != 0;
}

RTS_INLINE int RTProtocol::sendTypeCheckEnabled( void ) const
{
    return ( getFlags() & SendTypeCheck ) != 0;
}

RTS_INLINE int RTProtocol::receiveTypeCheckEnabled( void ) const
{
    return ( getFlags() & ReceiveTypeCheck ) != 0;
}

RTS_INLINE int RTProtocol::registerSAP( const char * serviceName )
{
    return registerSAP( serviceName, Programmatic );
}

RTS_INLINE int RTProtocol::deregisterSAP( void )
{
    return deregisterSAP( Programmatic );
}

RTS_INLINE int RTProtocol::registerSPP( const char * serviceName )
{
    return registerSPP( serviceName, Programmatic );
}

RTS_INLINE int RTProtocol::deregisterSPP( void )
{
    return deregisterSPP( Programmatic );
}

#endif // __RTProtocol_inl__
