/*
 * Licensed Materials - Property of IBM                               
 * Rational Software Architect RealTime Edition - TARGET RTS                  
 * Copyright IBM Corporation 1999, 2014. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTProtocolDescriptor_h__
#define __RTProtocolDescriptor_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTVersionId_h__
#include <RTVersionId.h>
#endif

class  RTGlobalSignal;
struct RTObject_class;
struct RTSignalDescriptor;

// A protocol is described by a pair of these structures: one for each of
// the conjugated and unconjugated perspectives. All signals, including
// ones inherited but not those excluded, are counted and listed in order
// sorted by name.
//
// Field           Meaning
// -----           -------
// super           The superclass descriptor.
// conjugate       The conjugate descriptor.
// name            The name of the protocol.
// version         The version identifier.
// numInSignals    The number of 'in' signals.
//                 The 'out' signals are listed in the conjugate.
// inSignals       The array of signal descriptors.

struct RTProtocolDescriptor
{
    const RTProtocolDescriptor * super;
    const RTProtocolDescriptor * conjugate;
    const char                 * name;
    RTVersionId                  version;

    int                          numInSignals;
    const RTSignalDescriptor   * inSignals;
    // 'out' signals are listed in conjugate->inSignals

    const char    * inSignalName( int signal ) const;
    int             inSignalNamed( const char * ) const;

    RTS_INLINE
    const char    * outSignalName( int signal ) const;
    RTS_INLINE
    int             outSignalNamed( const char * ) const;

    enum Status
    {
	ok
      , badSignal
      , badType
    };

    Status          compare( const RTProtocolDescriptor * ) const;
    RTS_INLINE int  compatibleWith( const RTProtocolDescriptor * ) const;

    RTS_INLINE
    Status          canSend( int signal, const RTObject_class * type ) const;
    Status          canReceive( int signal, const RTObject_class * type ) const;
};

#if RTS_INLINES
#include <RTProtocolDescriptor.inl>
#endif

#endif // __RTProtocolDescriptor_h__
