/*
 * Licensed Materials - Property of IBM                               
 * Rational Software Architect RealTime Edition - TARGET RTS                  
 * Copyright IBM Corporation 1999, 2014. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTController.h>
#include <RTMessage.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>
#include <RTProtocolDescriptor.h>

int RTProtocol::invoke( RTMessage            * msg,
			int                    signal,
			const void           * data,
			const RTObject_class * type,
			bool implicitReply)
{
    RTController * invoker = context();

    invoker->trace( RTController::CommInvoke, this, (const void *)0 );

#if RTTYPECHECK_SEND != RTTYPECHECK_DONT
    switch( sendCheck( signal, type ) )
    {
    case RTProtocolDescriptor::badSignal:
	invoker->setError( RTController::badSignal,
			   RTController::CommInvoke, this, (const void *)0 );
#if RTTYPECHECK_SEND == RTTYPECHECK_FAIL
	return 0;
#else
	break;
#endif

    case RTProtocolDescriptor::badType:
	invoker->setError( RTController::badClass,
			   RTController::CommInvoke, this, (const void *)0 );
#if RTTYPECHECK_SEND == RTTYPECHECK_FAIL
	return 0;
#else
	break;
#endif

    case RTProtocolDescriptor::ok:
    default:
	break;
    }
#endif // RTTYPECHECK_SEND != RTTYPECHECK_DONT

    int   result      = 0;
    int   replication = _info->replication;
    End * end         = _ends;

    for( int index = 0; index < replication; ++index, ++end )
	result += end->invoke( msg++, invoker, this, index,
			       signal, data, type, implicitReply );

    return result;
}
